# coding: utf-8

import click
import logging
from deployv_addon_gitlab_tools.commands import (
    check_keys, test_images, upload_image, deployv_tests, push_coverage,
    build_image, test_image, check_log
)

__version__ = "0.2.39"
_logger = logging.getLogger('deployv.' + __name__)


@click.group(invoke_without_command=True)
@click.pass_context
def gitlab_tools(ctx):
    """Deployv addon: Gitlab tools.
    """
    ctx.command.config = ctx.parent.command.config
    parent = ctx.parent.params
    _logger.log(
        getattr(logging, parent.get('log_level', 'INFO')),
        "Deployv addon: Gitlab tools (ver: %s).",
        __version__
    )
    if not ctx.invoked_subcommand:
        click.echo(ctx.get_help())


for command in [
        check_keys.check_keys,
        test_images.test_images,
        upload_image.upload_image,
        deployv_tests.deployv_tests,
        push_coverage.push_coverage,
        build_image.build_image,
        test_image.test_image,
        check_log.check_log

]:
    gitlab_tools.add_command(command)


@click.command()
def cli():
    click.echo("You can use 'Deployv addon: Gitlab Tools' from 'deployvcmd'.")
    click.echo("Run 'deployvcmd gitlab_tools --help' for more information.")



# coding: utf-8

from deployv_addon_gitlab_tools.common import check_env_vars, prepare
from deployv.helpers import utils
from docker import APIClient as Client
from os import environ, unsetenv
import subprocess
import shlex
import sys
import click
import logging
import json
import requests


_logger = logging.getLogger('deployv.' + __name__)
_cli = Client(timeout=600)


def build_image():
    format_values = {
        'url': environ['CI_REPOSITORY_URL'],
        'version': environ['CI_COMMIT_REF_NAME'],
        'base': environ['BASE_IMAGE'],
        'odoo_repo': environ['ODOO_REPO'],
        'odoo_branch': environ['ODOO_BRANCH'],
        'name': environ['_IMAGE_NAME'],
    }
    _logger.info('Bulding image %s', environ['_IMAGE_NAME'])
    cmd = (
        'deployvcmd build -u {url} -v {version} -i {base} -O {odoo_repo}#{odoo_branch} -T {name}'
        .format(**format_values)
    )
    subprocess.check_call(shlex.split(cmd))
    image_sha = _cli.images(name=environ['_IMAGE_NAME'], quiet=True)
    res = image_sha and image_sha[0].decode().split(':')[1][:10]
    return res


def push_image(tags):
    if environ.get('https_proxy', False):
        unsetenv('https_proxy')
    for tag in tags:
        _logger.info('Pushing image %s to %s:%s', environ['_IMAGE_NAME'], environ['_IMAGE_REPO'], tag)
        _cli.tag(environ['_IMAGE_NAME'], environ['_IMAGE_REPO'], tag=tag)
        for result in _cli.push(environ['_IMAGE_REPO'], tag=tag, stream=True):
            result = json.loads(utils.decode(result))
            if result.get('error'):
                _logger.error(result.get('error'))
                sys.exit(1)


def notify_orchest(tag, customer, is_latest=False):
    image_name = '{image}:{tag}'.format(image=environ['_IMAGE_REPO'],
                                        tag=tag)
    res = requests.post(
        environ['ORCHEST_REGISTRY'], data=json.dumps({
            'image_name': image_name, 'is_latest': is_latest, 'branch_name': environ['CI_COMMIT_REF_NAME'],
            'job_id': environ['CI_JOB_ID'], 'project_id': environ['CI_PROJECT_ID'],
            'commit': environ['CI_COMMIT_SHA'][:7], 'customer_id': customer}),
        headers={'Content-Type': 'application/json', 'Orchest-Token': environ['ORCHEST_TOKEN']})
    if res.status_code != 200:
        _logger.error('Failed to notify orchest about the new image: %s', res.text)
        sys.exit(1)
    data = res.json()
    if data.get('error'):
        _logger.error('Failed to notify orchest about the new image: %s',
                      data.get('error').get('data', {}).get('name'))
        sys.exit(1)
    _logger.info('Successfully notified orchest about the new image: %s', image_name)


@click.command()
@click.option('--ci_project_name', default=environ.get('CI_PROJECT_NAME'),
              help=("The project name that is currently being built."
                    " Env var: CI_PROJECT_NAME."))
@click.option('--CI_COMMIT_SHA', default=environ.get('CI_COMMIT_SHA'),
              help=("The commit revision for which project is built."
                    " Env var: CI_COMMIT_SHA."))
@click.option('--CI_COMMIT_REF_NAME', default=environ.get('CI_COMMIT_REF_NAME'),
              help=("The branch or tag name for which project is built."
                    " Env var: CI_COMMIT_REF_NAME."))
@click.option('--CI_REPOSITORY_URL', default=environ.get('CI_REPOSITORY_URL'),
              help=("The URL to clone the Git repository."
                    " Env var: CI_REPOSITORY_URL."))
@click.option('--base_image', default=environ.get('BASE_IMAGE'),
              help=("Env var: BASE_IMAGE."))
@click.option('--odoo_repo', default=environ.get('ODOO_REPO'),
              help=("Env var: ODOO_REPO."))
@click.option('--odoo_branch', default=environ.get('ODOO_BRANCH'),
              help=("Env var: ODOO_BRANCH."))
@click.option('--image_repo_url', default=environ.get('IMAGE_REPO_URL', "quay.io/vauxoo"),
              help=("The URL where the image repository is located."
                    " Env var: IMAGE_REPO_URL."))
@click.option('--orchest_registry', default=environ.get('ORCHEST_REGISTRY'),
              help=("Env var: ORCHEST_REGISTRY."))
@click.option('--orchest_token', default=environ.get('ORCHEST_TOKEN'),
              help=("Env var: ORCHEST_TOKEN."))
def upload_image(**kwargs):
    config = prepare(**kwargs)
    customer = environ.get('CUSTOMER', environ.get('CI_PROJECT_NAME'))
    version_tag = environ.get('VERSION').replace('.', '')
    image_name = '{customer}_{ver}'.format(
        customer=customer.replace(' ', '').replace(',', '_'),
        ver=version_tag
    )

    environ.update({'_IMAGE_NAME': image_name})
    image_sha = build_image()
    tags = [image_sha]
    is_latest = False
    if environ.get('CI_COMMIT_REF_NAME') == environ.get('VERSION'):
        tags.append('latest')
        is_latest = True
    customer_img = '{customer}{ver}'.format(customer=customer.strip(),
                                            ver=version_tag)
    image_repo = '{url}/{image}'.format(url=environ.get('IMAGE_REPO_URL'),
                                        image=customer_img)
    environ.update({'_IMAGE_REPO': image_repo})
    push_image(tags)
    notify_orchest(image_sha, customer_img, is_latest=is_latest)
    sys.exit(0)



import future
from os import environ
from os.path import isfile, isdir
import re
from sys import exit


CHECK = u'\033[92mV\033[0m '
CROSS = u'\033[91mX\033[0m '
CIRCLE = u'\033[1;33;40mO\033[0m '


def read_varsfile():
    res = []
    with open('variables.sh', 'r') as varsfile:
        lines = varsfile.readlines()
    for line in lines:
        cLine = line.strip()
        if cLine:
            res.append(cLine)
    return res


def get_file_values():
    rx = re.compile(r'^export\s(?P<key>\w.+?)(\s?)=(\s?)\"?(?P<value>[^\"]*)\"?')
    lines = read_varsfile()
    res = {}
    for line in lines:
        matchs = rx.match(line)
        if matchs:
            res.update({
                matchs.group('key'): matchs.group('value')
            })
    return res


def check_varsfile_syntax():
    nLine = 0
    rx = re.compile(r'^export\s\w*=.*')
    lines = read_varsfile()
    for line in lines:
        nLine += 1
        if not rx.match(line):
            print(CROSS + 'Syntax error in line {} variables file: {}'.format(nLine, line))
            exit(1)
    print(CHECK + 'File syntax OK')


def check_varsfile():
    if isfile('variables.sh'):
        print(CHECK + 'Variables file present')
    else:
        print(CROSS + 'Variables file not present, please create and add it to the repository in order to continue.' +
              ' It should be a list of bash exports like: export VERSION=12.0 or export VERSION="12.0"  (one per line)')
        exit(1)


def check_required_envars():
    required = ['VERSION', 'TRAVIS_PYTHON_VERSION', ]
    fail = False
    for v in required:
        if v in environ.keys():
            print(CHECK + 'Variable {} declared in environment'.format(v))
        else:
            print(CROSS + 'Variable {} not declared in environment,'.format(v) +
                  ' please define it to continue in the variables files.' +
                  '\nRemember to use bash syntax. Example: export VERSION=12.0 or export VERSION="12.0"  (one per line)')
            fail = True
    if fail:
        exit(1)


def check_envars():
    fValues = get_file_values()
    failed = []

    for k in fValues:
        if environ.get(k) != fValues[k]:
            failed.append('Failed check for variable: {}'.format(k))
    if failed:
        for f in failed:
            print(CROSS + f)
        print(CROSS + 'Values in your environment don\'t match the ones for variables.sh:' +
              '\n- did you sourced the variables.sh file? if no add "source variables.sh" to your ci file' +
              ' (should be the first line in the script section)' +
              '\n- do you have a repeated value for a particular variable in variables.sh and' +
              ' in the variables section in your ci file?' +
              ' if so please remove the one from the ci file and use it in variables.sh only')
        exit(1)
    else:
        print(CHECK + 'Env vars match your variables.sh')


def check_mainapp():
    fValues = get_file_values()
    main_app = fValues.get('MAIN_APP', False)
    if not main_app:
        print(CIRCLE + 'No main app defined, all modules will be installed')
        return
    apps = main_app.split(',')
    fail = False
    for app in apps:
        if isdir(app):
            print(CHECK + 'Main app "{}" exists and match the one from your vars'.format(app))
        else:
            print(CROSS + 'Got {} from your env vars, but it doesn\'t match any folder in you repo'.format(app) +
                  ' Remember that the MAIN_APP is the app that will be installed and tested,' +
                  ' this have to match one Odoo module in this repository')
            fail = True
    if fail:
        exit(1)


def check_all():
    check_varsfile()
    check_varsfile_syntax()
    check_envars()
    check_required_envars()
    check_mainapp()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

import io
import re
import os
import sys
import errno
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup
from setuptools.command.install import install as InstallCommand
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


def _fix_ownership(path):
    uid = os.environ.get('SUDO_UID')
    gid = os.environ.get('SUDO_GID')
    if uid is not None:
        os.chown(path, int(uid), int(gid))


class PyInstall(InstallCommand):

    def run(self):
        InstallCommand.run(self)
        self.setup_config_dirs()

    def setup_config_dirs(self):
        sys.stdout.write("Writing config files\n")
        default_config_file = 'data/gitlab_tools.conf'
        for main_dir in ['/etc/deployv', os.path.expanduser('~/.config/deployv')]:
            addon_dir = os.path.join(main_dir, 'conf.d')
            config_file_name = os.path.join(addon_dir, 'gitlab_tools.conf')
            error_msg = "Couldn't write config file: '{file}' (skipped)\n"
            ok_msg = "Created config file: '{file}'\n"
            is_user_dir = os.path.expanduser('~/') in main_dir
            try:
                os.makedirs(addon_dir)
            except OSError as error:
                if error.errno != errno.EEXIST:
                    sys.stdout.write(error_msg.format(file=config_file_name))
                    continue
            else:
                if is_user_dir:
                    for path in [main_dir.replace('/deployv', ''), main_dir, addon_dir]:
                        _fix_ownership(path)
            config = ConfigParser()
            config.read([default_config_file, config_file_name])
            try:
                with open(config_file_name, 'w+') as config_file:
                    config.write(config_file)
            except (OSError, IOError) as error:
                sys.stdout.write(error_msg.format(file=config_file_name))
            else:
                sys.stdout.write(ok_msg.format(file=config_file_name))
                if is_user_dir:
                    _fix_ownership(config_file_name)


setup(
    name='deployv-addon-gitlab-tools',
    version='0.2.39',
    license='BSD',
    description='Deployv addon: Gitlab tools. Generated by cookiecutter and cookiecutter-deploy-addon.',
    long_description='%s\n%s' % (
        re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).sub('', read('README.rst')),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    author='Vauxoo',
    author_email='info@vauxoo.com',
    url='https://github.com/Vauxoo/deployv-addon-gitlab-tools',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    package_data={'': ['data/*']},
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        # uncomment if you test on these interpreters:
        # 'Programming Language :: Python :: Implementation :: IronPython',
        # 'Programming Language :: Python :: Implementation :: Jython',
        # 'Programming Language :: Python :: Implementation :: Stackless',
        'Topic :: Utilities',
    ],
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    install_requires=[
        'deployv',
        'future'
    ],
    extras_require={
        # eg:
        #   'rst': ['docutils>=0.11'],
        #   ':python_version=="2.6"': ['argparse'],
    },
    cmdclass={
        'install': PyInstall,
    },
    entry_points={
        'console_scripts': [
            'deployv-addon-gitlab-tools = deployv_addon_gitlab_tools:cli',
        ]
    },
)
